@@ -10,16 +10,16 @@ die () {
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.27-59vl5.src.rpm ]
+if [ ! -r kernel-2.6.27-65vl5.src.rpm ]
 then
-    wget http://updates.vinelinux.org/Vine-5.1/updates/SRPMS/kernel-2.6.27-59vl5.src.rpm || die "Can't download source package."
+    wget http://updates.vinelinux.org/Vine-5.1/updates/SRPMS/kernel-2.6.27-65vl5.src.rpm || die "Can't download source package."
 fi
-rpm -ivh kernel-2.6.27-59vl5.src.rpm || die "Can't install source package."
+rpm -ivh kernel-2.6.27-65vl5.src.rpm || die "Can't install source package."
 
 cd /usr/src/vine/SOURCES/ || die "Can't chdir to /usr/src/vine/SOURCES/ ."
-if [ ! -r ccs-patch-1.6.8-20100923.tar.gz ]
+if [ ! -r ccs-patch-1.6.8-20101122.tar.gz ]
 then
-    wget http://sourceforge.jp/frs/redir.php?f=/tomoyo/30297/ccs-patch-1.6.8-20100923.tar.gz || die "Can't download patch."
+    wget -O ccs-patch-1.6.8-20101122.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/30297/ccs-patch-1.6.8-20101122.tar.gz' || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -28,11 +28,11 @@ patch << "EOF" || die "Can't patch spec file."
 --- kernel-2.6-vl.spec
 +++ kernel-2.6-vl.spec
 @@ -27,7 +27,7 @@
- %define patchlevel 51
+ %define patchlevel 54
  %define kversion 2.6.%{sublevel}
  %define rpmversion 2.6.%{sublevel}
--%define release 59%{?_dist_release}
-+%define release 59%{?_dist_release}_tomoyo_1.6.8p3
+-%define release 65%{?_dist_release}
++%define release 65%{?_dist_release}_tomoyo_1.6.8p4
  
  %define make_target bzImage
  %define hdrarch %_target_cpu
@@ -55,18 +55,18 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  Version: %{rpmversion}
-@@ -787,6 +790,10 @@
+@@ -804,6 +807,10 @@
  
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.6.8-20100923.tar.gz
++tar -zxf %_sourcedir/ccs-patch-1.6.8-20101122.tar.gz
 +patch -sp1 < patches/ccs-patch-2.6.27-vine-linux-5.1.diff
 +
  cp %{SOURCE10} Documentation/
  
  # put Vine logo
-@@ -805,6 +812,9 @@
+@@ -822,6 +829,9 @@
  for i in *.config
  do 
  	mv $i .config 
